/**
   \file
   \ingroup    SASUNIT_ASSERT 

   \brief      Check whether a certain message appears in the log.

               If the message does not appear in the log as expected, 
               the check of the assertion will fail.
               If i_not is set to 1, the check of the assertion will fail in case the 
               message is found in the log.

   \version    \$Revision: 316 $
   \author     \$Author: akirchmann $
   \date       \$Date: 2014-03-02 10:20:58 +0100 (So, 02 Mrz 2014) $
	\sa			For further details refer to <A href="https://sourceforge.net/p/sasunit/wiki/User's%20Guide/" target="_blank">SASUnit User's Guide</A>. 
					Here you can find the SASUnit documentation, release notes and license information.
   \sa         \$HeadURL: https://svn.code.sf.net/p/sasunit/code/trunk/saspgm/sasunit/assertlogmsg.sas $
   \copyright  Copyright 2010, 2012 HMS Analytical Software GmbH.
               This file is part of SASUnit, the Unit testing framework for SAS(R) programs.
               For terms of usage under the GPL license see included file readme.txt
               or https://sourceforge.net/p/sasunit/wiki/readme.v1.2/.

   \param   i_logmsg       message of interest (Perl Regular Expression), non-case-sensitive log scan,
                           special characters have to be quoted with a prefixed single backslash,
                           see http://support.sas.com/onlinedoc/913/getDoc/de/lrdict.hlp/a002288677.htm#a002405779
   \param   i_desc         description of the assertion to be checked \n
                           default: "Scan for log messages"
   \param   i_not          negates the assertion, if set to 1
   
*/ /** \cond */ 

%MACRO assertLogMsg (i_logmsg   =       
                    ,i_desc     = Scan for log messages  
                    ,i_not      = 0
                    );

   /*-- verify correct sequence of calls-----------------------------------------*/
   %GLOBAL g_inTestcase;
   %IF &g_inTestcase EQ 1 %THEN %DO;
      %endTestcall;
   %END;
   %ELSE %IF &g_inTestcase NE 2 %THEN %DO;
      %PUT &g_error.(SASUNIT): assert must be called after initTestcase;
      %RETURN;
   %END;

   %LOCAL l_casid l_msg_found l_actual l_expected l_assert_failed;
   PROC SQL NOPRINT;
   /* determine number of the current test case */
      SELECT max(cas_id) INTO :l_casid FROM target.cas WHERE cas_scnid = &g_scnid;
   QUIT;

   %IF &l_casid = . OR &l_casid = %THEN %DO;
      %PUT &g_error.(SASUNIT): assert must not be called before initTestcase;
      %RETURN;
   %END;

   /* scanne den Log */
   %LET l_msg_found=0;
   DATA _null_;
      RETAIN pattern_id;
      IF _n_=1 THEN DO;
         pattern_id = prxparse("/%upcase(&i_logmsg)/");
      END;
      INFILE "&g_log/%sysfunc(putn(&g_scnid,z3.))_%sysfunc(putn(&l_casid,z3.)).log" END=eof TRUNCOVER;
      INPUT logrec $char256.;
      logrec = upcase(logrec);
      IF prxmatch (pattern_id, logrec) THEN DO;
         call symput ('l_msg_found', '1');
      END;
   RUN;

   %IF &l_msg_found %THEN %DO;
      %LET l_actual = 1; /* message found */
   %END;
   %ELSE %DO;
      %LET l_actual = 2; /* message not found */
   %END;

   %IF &i_not %THEN %DO;
      %LET l_expected = 2&i_logmsg; /* message not present */
      %LET l_assert_failed = %eval (&l_msg_found.*2);
   %END;
   %ELSE %DO;
      %LET l_expected = 1&i_logmsg; /* message present */
      %LET l_assert_failed = %eval((NOT &l_msg_found)*2);
   %END;

   %_asserts(i_type     = assertLogMsg
            ,i_expected = %str(&l_expected)
            ,i_actual   = %str(&l_actual)
            ,i_desc     = &i_desc
            ,i_result   = &l_assert_failed
            )

%MEND assertLogMsg;
/** \endcond */
